"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.par = void 0;
const debug_1 = require("../../debug");
const log_1 = require("../../log");
const types_1 = require("../types");
const command_1 = require("./command");
const group_utility_1 = require("./group-utility");

/** Converts `command` to a `Promise` to be passed to `Promise.all()`. */
const runCommandInPar = (onEvent) => (command) =>
  new Promise((resolve) =>
    command.run(onEvent).then(
      () => resolve(false),
      (e) => {
        log_1.error(e);
        resolve(true);
      }
    )
  );

/** `run()` method for `ParallelCommand`. */
const runPar = async function (onEvent) {
  const startResult = onEvent(this, types_1.Event.Start);
  if (startResult) return startResult;
  const promises = this.children.map(runCommandInPar(onEvent));
  const errors = await Promise.all(promises);
  let anyError = false;
  for (const err of errors) {
    if (err) {
      anyError = true;
      const onFailureResult = onEvent(this, types_1.Event.Failure);
      if (onFailureResult) return onFailureResult;
      if (!this.ignoresFailure)
        return Promise.reject("Found error in parallel commands.");
    }
  }
  if (!anyError) return onEvent(this, types_1.Event.Success);
};

/** Emits debug log. */
const inspectCommands = (commands) => {
  if (!debug_1.logDebug) return;
  debug_1.debug("prepare parallel:");
  debug_1.debugLines(group_utility_1.getCommandNames(commands), "  ");
};

/** Creates a `Command` object that runs given commands in parallel. */
const par = (...commands) => {
  const children = group_utility_1.normalizeCommands(commands);
  inspectCommands(children);
  const base = command_1.createCommand({
    run: runPar,
    type: types_1.CommandType.Group,
    subType: types_1.CommandSubType.Parallel,
    name: "",
  });
  return Object.assign(base, { children });
};
exports.par = par;
