/** Value that specifies the type of a `Command` object. */
export declare const CommandType: {
  readonly Unit: "unit";
  readonly Group: "group";
};
export declare type CommandType = typeof CommandType[keyof typeof CommandType];

/** Value that specifies the sub-type of a `Command` object. */
export declare const CommandSubType: {
  readonly CommandLine: "commandline";
  readonly External: "external";
  readonly Null: "null";
  readonly Sequence: "sequence";
  readonly Parallel: "parallel";
};
export declare type CommandSubType = typeof CommandSubType[keyof typeof CommandSubType];

/** Event fired by `Command` objects. */
export declare const Event: {
  readonly Start: "start";
  readonly Success: "success";
  readonly Failure: "failure";
};
export declare type Event = typeof Event[keyof typeof Event];

/**
 * Function for handling command events.
 * If it returns a `Promise`, it overrides the default behavior of `command`.
 */
export declare type EventHandler = (
  command: Command,
  event: Event
) => Promise<void> | undefined;
export declare const DisplayState: {
  readonly Collapsed: "collapsed";
  readonly Hidden: "hidden";
};
export declare type DisplayState = typeof DisplayState[keyof typeof DisplayState];

/** Command object. Can be wrapped with `seq()` or `par()`. */
export interface Command {
  readonly run: (onEvent: EventHandler) => Promise<void>;
  readonly type: CommandType;
  readonly subType: CommandSubType;
  readonly children?: readonly Command[];
  readonly ignoresFailure: boolean;
  readonly ignoreFailure: () => Command;
  readonly name: string;
  readonly rename: (name: string) => Command;
  readonly displayState: DisplayState | undefined;
  readonly hide: () => Command;
  readonly collapse: () => Command;
}

/** Data unit for recording fired events. */
export interface EventRecord {
  readonly event: Event;
  readonly timestamp: number;
}

/** Return value from `run()`. */
export declare type Result = Map<Command, EventRecord[]>;

/** Object that runs different commands depending on the given key. */
export interface Router {
  readonly run: (
    key: string,
    onEvent?: (command: Command, event: Event) => any,
    onSuccessAll?: () => any,
    onFailureAny?: () => any
  ) => Promise<Result>;
  readonly help: () => void;
}
